unit BBoard;

(*--------------------------------------------------------------------*)
(*																																		*)
(* TBillBoard Beta Release 1.0																				*)
(*																																		*)
(* author	:	Eduardo Manso Neto ( eduardomanso@netsite.com.br )				*)
(*																																		*)
(* 					Copyright 1997 MANSOFT Informtica Ltda										*)
(*																																		*)
(*--------------------------------------------------------------------*)

interface

uses

  Windows, SysUtils, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, DsgnIntf,
  BBConst, BBCommon;

(*											*)
(* Forward declaration	*)
(*											*)

procedure Register;

(*																		*)
(* TBBMask property editor definition	*)
(*																		*)

type

	TBBMaskPropertyEditor = class( TClassProperty )

    public
    	procedure Edit; override;
      function 	GetAttributes: TPropertyAttributes; override;
      function 	GetValue: string; override;

	end;

(*																				*)
(* TBillBoard Component Editor definition	*)
(*																				*)

type

  TBillBoardComponentEditor = class( TComponentEditor )

    protected
      procedure ShowAbout;
      procedure ShowEditor;

    public
      procedure Edit; override;
      procedure ExecuteVerb( Index: Integer ); override;
      function GetVerb( Index: Integer ): string; override;
      function GetVerbCount: Integer; override;

  end;

(*																*)
(* 	TBillBoard control definition	*)
(*																*)

type

	TDotStyle = ( dsDot, dsBulb );

  TBillBoard = class( TGraphicControl )
    private
      FAnimation : boolean;
      FAnimationRate : integer;
      FBackColor : TColor;
      FBackGndBmp: TBitmap;
	    FBBMask : TBBMask;
      FCellSize : integer;
      FChanged: Boolean;
      FCharHeight : integer;
      FCharWidth : integer;
	    FCustomMask : Boolean;
      FDotOffColor : TColor;
      FDotOnColor : TColor;
      FDotSize : integer;
    	FDotStyle	: TDotStyle;
      FOldHeight: integer;
      FOldWidth: integer;
      FOnBmp: TBitmap;
      FPenOffColor : TColor;
      FPenOnColor : TColor;
      FStartCell: integer;
      FStartPoint: TPoint;
      FText : string;
      FTimer : TTimer;
      procedure DrawBackGround( Bitmap: TBitmap );
      procedure DrawDigit( Digit: Char; FromCell: TPoint; Bitmap: TBitmap );
      procedure DrawDynamicText( Text: string; Bitmap: TBitmap );
      procedure DrawStaticText( Text: string; Cell: TPoint; Bitmap: TBitmap );
      procedure SetAnimationOnOff( Value: boolean );
      procedure SetAnimationRate( Value: integer );
      procedure SetBackColor( Value: TColor );
      procedure SetCharset( Value: TBBMask );
      procedure SetDotOffColor( Value: TColor );
      procedure SetDotSize( Value: integer );
      procedure SetDotStyle( dotStyle: TDotStyle );
      procedure SetDotOnColor( Value: TColor );
      procedure SetPenOffColor( Value: TColor );
      procedure SetPenOnColor( Value: TColor );
      procedure SetSizes;
      procedure SetText( Value: string );
    protected
			FOnClick: TNotifyEvent;
			FOnDblClick: TNotifyEvent;
			FOnNeedText: TNotifyEvent;
      FOnMouseDown: TMouseEvent;
      FOnMouseUp: TMouseEvent;
      FOnMouseMove: TMouseMoveEvent;
			procedure OnTimer( Sender: TObject );
      procedure DefineProperties( Filer: TFiler ); override;
      procedure ReadCustomMask( reader: TReader );
      procedure WriteCustomMask( writer: TWriter );
      procedure ReadCustom( stream: TStream );
      procedure WriteCustom( stream: TStream );
      procedure Paint; override;
			procedure Click; override;
			procedure DblClick; override;
			procedure MouseDown( Button: TMouseButton; Shift: TShiftState; X,Y: integer ); override;
			procedure MouseUp( Button: TMouseButton; Shift: TShiftState; X,Y: integer ); override;
 			procedure MouseMove( Shift: TShiftState; X,Y: integer ); override;
    public
      constructor Create( AOwner: TComponent ); override;
      destructor Destroy; override;
      property CustomCharset: Boolean read FCustomMask write FCustomMask default false;
      property CellSize : integer read FCellSize;
      property CharWidth : integer read FCharWidth;
    published
      property Align;
      property Tag;
      property Animation : boolean read FAnimation write SetAnimationOnOff default false;
      property AnimationRate : integer read FAnimationRate write SetAnimationRate default 500;
      property BackColor : TColor read FBackColor write SetBackColor default clBlack;
			property Charset : TBBMask read FBBMask write SetCharset;
      property DotOffColor : TColor read FDotOffColor write SetDotOffColor default clBlack;
      property DotOnColor : TColor read FDotOnColor write SetDotOnColor default clLime;
      property DotSize : integer read FDotSize write SetDotSize default 4;
      property DotStyle : TDotStyle read FDotStyle write SetDotStyle default dsDot;
      property OnClick : TNotifyEvent read FOnClick write FOnClick;
      property OnDblClick : TNotifyEvent read FOnDblClick write FOnDblClick;
      property OnNeedText : TNotifyEvent read FOnNeedText write FOnNeedText;
      property OnMouseDown : TMouseEvent read FOnMouseDown write FOnMouseDown;
      property OnMouseMove : TMouseMoveEvent read FOnMouseMove write FOnMouseMove;
      property OnMouseUp : TMouseEvent read FOnMouseUp write FOnMouseUp;
      property PenOffColor : TColor read FPenOffColor write SetPenOffColor default clTeal;
      property PenOnColor : TColor read FPenOnColor write SetPenOnColor default clRed;
      property Text : string read FText write SetText;

  end;

implementation

uses

	BBAbout, BBEditor;

(*																				*)
(* TBBMask property editor implementation	*)
(*																				*)

procedure TBBMaskPropertyEditor.Edit;
var
	bb: TBillBoard;
  ed: TBillBoardEditor;
begin

	if( PropCount > 0 ) then
  	begin
    	ed := TBillBoardEditor.Create( Application );
      try
      	bb := TBillBoard( GetComponent( 0 ) );
        ed.Execute( bb );
        bb.Refresh;
      finally
      	ed.Free;
      end;
    end;

end;

function TBBMaskPropertyEditor.GetAttributes: TPropertyAttributes;
begin

	Result := [ paMultiSelect, paDialog ];

end;

function TBBMaskPropertyEditor.GetValue: string;
var
	bb: TBillBoard;
begin

	if( PropCount > 0 ) then
  	begin
      try
        bb := TBillBoard( GetComponent( 0 ) );
        if( not( bb.CustomCharset ) ) then
        	Result := 'Default'
        else
        	Result := 'Custom';
      except
      	Result := '?';
      end;
    end;

end;

(*																						*)
(* TBillBoard Component Editor implementation	*)
(*																						*)

function TBillBoardComponentEditor.GetVerbCount: Integer;
begin

  Result := 2;

end;

function TBillBoardComponentEditor.GetVerb( Index: Integer ): string;
begin

	case Index of
  	0: Result := 'About...';
    1: Result := 'Edit charset...';
  end;

end;

procedure TBillBoardComponentEditor.Edit;
begin

  ShowEditor;

end;

procedure TBillBoardComponentEditor.ExecuteVerb( Index: Integer );
begin

	case Index of
  	0: ShowAbout;
    1: ShowEditor;
  end;

end;

procedure TBillBoardComponentEditor.ShowAbout;
var
	bba: TBillBoardAboutBox;
begin

	bba := TBillBoardAboutBox.Create( Application );
	try
    bba.ShowModal;
  finally
    bba.Free;
  end;

end;

procedure TBillBoardComponentEditor.ShowEditor;
var
	bb: TBillBoard;
  ed: TBillBoardEditor;
begin

	ed := TBillBoardEditor.Create( Application );
  try
    bb := TBillBoard( Component );
    ed.Execute( bb );
    bb.Refresh;
  finally
    ed.Free;
  end;

end;

(*																		*)
(* 	TBillBoard control implementation	*)
(*																		*)

constructor TBillBoard.Create( AOwner: TComponent );
begin

  inherited Create( AOwner );

  ControlStyle := ControlStyle + [ csOpaque ];
	FBBMask := TBBMask.Create( Self );
	FCustomMask := false;
  FDotStyle := dsDot;
  FBackColor := clBlack;
  FDotOnColor := clLime;
  FDotOffColor := clBlack;
  FPenOnColor := clRed;
  FPenOffColor := clTeal;
  FText := 'Mansoft Informtica';
  FDotSize := 4;
  SetSizes;
  Width := FCharWidth * Length( FText );
  Height := FCharHeight;
  FAnimationRate := 500;
  FChanged := true;

end;

destructor TBillBoard.Destroy;
begin

	if( Assigned( FOnBmp ) ) then
  	FOnBmp.Free;

	if( Assigned( FBackGndBmp ) ) then
  	FBackGndBmp.Free;

  inherited Destroy;

end;

procedure TBillBoard.DefineProperties( Filer: TFiler );
begin

	inherited DefineProperties( Filer );

  Filer.DefineProperty( 'CustomMask', ReadCustomMask, WriteCustomMask, true );
  Filer.DefineBinaryProperty( 'Custom', ReadCustom, WriteCustom, CustomCharset );

end;

procedure TBillBoard.ReadCustomMask( reader: TReader );
begin

	FCustomMask := reader.ReadBoolean;

end;

procedure TBillBoard.WriteCustomMask( writer: TWriter );
begin

	writer.WriteBoolean( FCustomMask );

end;

procedure TBillBoard.ReadCustom( stream: TStream );
begin

	FBBMask.Load( stream );

end;

procedure TBillBoard.WriteCustom( stream: TStream );
begin

	FBBMask.Save( stream );

end;

procedure TBillBoard.SetSizes;
begin

  FCellSize := FDotSize + ( GapWidth * 2 );
  FCharWidth := ( MaskCharWidth + GapWidth ) * FCellSize;
  FCharHeight := MaskCharHeight * FCellSize;

end;

procedure TBillBoard.SetCharset( Value: TBBMask );
begin

	FBBMask.Copy( Value );

end;

procedure TBillBoard.SetBackColor( Value: TColor );
begin

  if( Value <> FBackColor ) then
    begin
      FBackColor := Value;
      FChanged := true;
      Refresh;
    end;

end;

procedure TBillBoard.SetDotOnColor( Value: TColor );
begin

  if( Value <> FDotOnColor ) then
    begin
      FDotOnColor := Value;
      FChanged := true;
      Refresh;
    end;

end;

procedure TBillBoard.SetDotOffColor( Value: TColor );
begin

  if( Value <> FDotOffColor ) then
    begin
      FDotOffColor := Value;
      FChanged := true;
      Refresh;
    end;

end;

procedure TBillBoard.SetPenOnColor( Value: TColor );
begin

  if( Value <> FPenOnColor ) then
    begin
      FPenOnColor := Value;
      FChanged := true;
      Refresh;
    end;

end;

procedure TBillBoard.SetPenOffColor( Value: TColor );
begin

  if( Value <> FPenOffColor ) then
    begin
      FPenOffColor := Value;
      FChanged := true;
      Refresh;
    end;

end;

procedure TBillBoard.SetDotSize( Value: integer );
begin

  if( ( Value <> FDotSize ) and
  		( Value > 1 ) and
      ( Value <= 150 ) ) then
    begin
      FDotSize := Value;
      SetSizes;
      FChanged := true;
      Refresh;
    end;

end;

procedure TBillBoard.SetDotStyle( dotStyle: TDotStyle );
begin

	if( FDotStyle <> dotStyle ) then
  	begin
    	FDotStyle := dotStyle;
      FChanged := true;
      Refresh;
    end;

end;

procedure TBillBoard.SetAnimationOnOff( Value: boolean );
begin

	if( FAnimation <> Value ) then
  	FAnimation := Value;

  if( FAnimation ) then
  	begin
      FStartCell := ( Width div FCellSize ) + 1;
      FStartPoint := Point( FStartCell, 0 );
    	FTimer := TTimer.Create( Application );
      FTimer.OnTimer := OnTimer;
      FTimer.Interval := FAnimationRate;
      FTimer.Enabled := true;
    end
  else
  	begin
    	if( Assigned( FTimer ) ) then
      	begin
          FTimer.Enabled := false;
          FTimer.Free;
          FTimer := nil;
      		FStartPoint := Point( 0, 0 );
          Refresh;
        end;
    end;

end;

procedure TBillBoard.SetAnimationRate( Value: integer );
begin

	if( FAnimationRate <> Value ) then
  	begin
    	if( FAnimation ) then
      	FTimer.Enabled := false;
  		FAnimationRate := Value;
    	if( FAnimation ) then
      	begin
        	FTimer.Interval := Value;
      		FTimer.Enabled := true;
        end;
    end;

end;

procedure TBillBoard.SetText( Value: string );
begin

	if( FText <> Value ) then
    begin
      FText := Value;
      if( FAnimation ) then
        begin
          FAnimation := false;
          FAnimation := true;
        end
      else
      	Refresh;
    end;

end;

procedure TBillBoard.Click;
begin

	inherited Click;

	if( Assigned( FOnClick ) ) then
  	FOnClick( Self );

end;

procedure TBillBoard.DblClick;
begin

	inherited DblClick;

	if( Assigned( FOnDblClick ) ) then
  	FOnDblClick( Self );

end;

procedure TBillBoard.MouseDown( Button: TMouseButton;
																	Shift: TShiftState; X,Y: integer );
begin

	inherited MouseDown( Button, Shift, X, Y );

	if( Assigned( FOnMouseDown ) ) then
  	FOnMouseDown( Self, Button, Shift, X, Y );

end;

procedure TBillBoard.MouseUp( Button: TMouseButton;
      													Shift: TShiftState; X,Y: integer );
begin

	inherited MouseUp( Button, Shift, X, Y );

	if( Assigned( FOnMouseUp ) ) then
  	FOnMouseUp( Self, Button, Shift, X, Y );

end;

procedure TBillBoard.MouseMove( Shift: TShiftState; X,Y: integer );
begin

	inherited MouseMove( Shift, X, Y );

	if( Assigned( FOnMouseMove ) ) then
  	FOnMouseMove( Self, Shift, X, Y );

end;

procedure TBillBoard.OnTimer( Sender: TObject );
begin

	if( Length( FText ) > 0 ) then
  	Refresh;

end;

procedure TBillBoard.Paint;
var
  R: TRect;
  Bitmap: TBitmap;
begin

  inherited Paint;

  if( ( FChanged ) or
  		( Width <> FOldWidth ) or
      ( Height <> FOldHeight ) ) then
  	begin
      R := ClientRect;
    	FBackGndBmp.Free;
      FBackGndBmp := TBitmap.Create;
      FBackGndBmp.Height := Height;
      FBackGndBmp.Width := Width;
      FBackGndBmp.Canvas.Brush.Color := FBackColor;
      FBackGndBmp.Canvas.FillRect( R );
      FOldHeight := Height;
      FOldWidth := Width;
      FChanged := true;
      DrawBackGround( FBackGndBmp );
    end;

  Bitmap := TBitmap.Create;
  Bitmap.Assign( FBackGndBmp );

  if( not( FAnimation ) ) then
  	DrawStaticText( FText, FStartPoint, Bitmap )
  else
  	DrawDynamicText( FText, Bitmap );

  Canvas.CopyMode := cmSrcCopy;
  Canvas.Draw( 0, 0, Bitmap );

  Bitmap.Free;

end;

procedure TBillBoard.DrawBackGround( Bitmap: TBitmap );
var
	col,row: integer;
  x,y,x1,y1: integer;
begin

  Bitmap.Canvas.Pen.Color := FPenOffColor;
  Bitmap.Canvas.Brush.Color := FDotOffColor;

  for row := 0 to ( Height div FDotSize ) + GapWidth do
    begin
      for col := 0 to ( Width div FDotSize ) + GapWidth do
        begin
          x := ( col * FCellSize ) + GapWidth;
          y := ( row * FCellSize ) + GapWidth;
          x1 := x + FCellSize - ( GapWidth * 2 );
          y1 := y + FCellSize - ( GapWidth * 2 );
          if( FDotStyle = dsBulb ) then
            Bitmap.Canvas.Ellipse( x, y, x1, y1 )
          else
            Bitmap.Canvas.Rectangle( x, y, x1, y1 );
        end;
    end;

end;

procedure TBillBoard.DrawDynamicText( Text: string; Bitmap: TBitmap );
var
	i: integer;
  p: TPoint;
begin

  if( FStartPoint.x < -( Length( Text ) * ( MaskCharWidth + GapWidth ) ) ) then
    begin
      if( Assigned( FOnNeedText ) ) then
        FOnNeedText( Self );
      FStartPoint.x := FStartCell;
    end;

  p := FStartPoint;
	if( Length( Text ) <> 0 ) then
  	begin
      for i := 1 to Length( Text ) do
      	begin
          DrawDigit( Text[ i ], p, Bitmap );
          inc( p.x, MaskCharWidth + GapWidth );
        end;
		end;
  dec( FStartPoint.x, GapWidth );

end;

procedure TBillBoard.DrawStaticText( Text: string; Cell: TPoint; Bitmap: TBitmap );
var
	i,j,k,m,n,r,x: integer;
begin

	if( Length( Text ) <> 0 ) then
  	begin
    	x := Cell.x;
    	n := 1;
      m := Width div FCharWidth;
      if( m = 0 ) then
      	m := 1;
			k := Length( Text ) div m;
      r := Length( Text ) mod m;
    	for i := 1 to k do
      	begin
          for j := 1 to m do
            begin
              DrawDigit( Text[ n ], Cell, Bitmap );
              inc( Cell.x, MaskCharWidth + GapWidth );
              inc( n );
            end;
        	inc( Cell.y, MaskCharHeight );
          Cell.x := x;
        end;
      if( r > 0 ) then
        begin
          for j := 1 to r do
            begin
              DrawDigit( Text[ n ], Cell, Bitmap );
              inc( Cell.x, MaskCharWidth + GapWidth );
              inc( n );
            end;
        end;
		end;

end;

procedure TBillBoard.DrawDigit( Digit: Char; FromCell: TPoint; Bitmap: TBitmap );
var
	row,line,maskbit: integer;
  x,y: integer;
  dgt: integer;
  r: TRect;
begin

  Bitmap.Canvas.Pen.Color := FPenOnColor;
  Bitmap.Canvas.Brush.Color := FDotOnColor;

  if( FChanged ) then
  	begin
      r := Rect( 0, 0, FCellSize, FCellSize );
    	FOnBmp.Free;
      FOnBmp := TBitmap.Create;
      FOnBmp.Height := FCellSize;
      FOnBmp.Width := FCellSize;
      FOnBmp.Canvas.Brush.Color := FBackColor;
      FOnBmp.Canvas.FillRect( r );
  		FOnBmp.Canvas.Pen.Color := FPenOnColor;
      FOnBmp.Canvas.Brush.Color := FDotOnColor;
      if( FDotStyle = dsBulb ) then
        FOnBmp.Canvas.Ellipse( GapWidth, GapWidth, FCellSize - GapWidth, FCellSize - GapWidth )
      else
        FOnBmp.Canvas.Rectangle( GapWidth, GapWidth, FCellSize - GapWidth, FCellSize - GapWidth );
      FChanged := false;
    end;

	dgt := Ord( Digit );

  for maskbit := MaskCharWidth - 1 downto 0 do
  	begin
  		row := FromCell.y;
      for line := 0 to MaskCharHeight - 1 do
        begin
          x := FromCell.x * FCellSize;
          y := row * FCellSize;
          if( ( FBBMask.Item[ dgt, line ] and ( $01 shl maskbit ) ) <> 0 ) then
          	Bitmap.Canvas.Draw( x, y, FOnBmp );
      		inc( row );
        end;
      inc( FromCell.x );
    end;

end;

(*									*)
(* Register all...	*)
(*									*)

procedure Register;
begin

  RegisterComponentEditor( TBillBoard, TBillBoardComponentEditor );

	RegisterPropertyEditor(	TypeInfo( TBBMask ), TBillBoard,
                          'Charset', TBBMaskPropertyEditor );

  RegisterComponents(	'Mansoft', [ TBillBoard ] );

end;

end.
